%******************************************************************************;
%* SAS macro TickMark vs. 0.0.1 , 25 Jul 2011, by Jim Groeneveld, Netherlands *;
%******************************************************************************;
%* STILL UNDER CONSTRUCTION, NOT FOR VALIDATION, USE WITH CAUTION AT OWN RISK *;
%******************************************************************************;
%*                                                                            *;
   %MACRO   TickMark   /* Neat automatic ticmarks for graphs based on data    */
/* -------------------------------------------------------------------------- */
/* ---------- A l w a y s   s p e c i f i e d   a r g u m e n t s ----------- */
/* Argument Default       Description and remarks      Required/Optional: R/O */
/* -------- --------      --------------------------------------------------- */
(  Minimum =           /* Minimum value of values to scale with tick marks  R */
,  Maximum =           /* Maximum value of values to scale with tick marks  R */
,  FromVal =           /* existing! macro variable to return From Value to  R */
,  ToValue =           /* existing! macro variable to return  To  Value to  R */
,  ByValue =           /* existing! macro variable to return  By  Value to  R */
/*                                                                            */
/* ------ F r e q u e n t l y   s p e c i f i e d   a r g u m e n t s ------- */
/* Argument Default       Description and remarks      Required/Optional: R/O */
/* -------- --------      --------------------------------------------------- */
,  MinTicks= 7         /* minimum number of desired tickmarks               O */
,  MaxTicks= 12        /* maximum number of desired tickmarks               O */
,  Coverage= 80        /* minimum percentage coverage of result value range O */
/*                                                                            */
/* ---------- R a r e l y   s p e c i f i e d   a r g u m e n t s ----------- */
/* Argument Default       Description and remarks      Required/Optional: R/O */
/* -------- --------      --------------------------------------------------- */
,  Version = 0.0.1     /* forced version specification for version control  O */
/*                                                                            */
/* ----------- N e v e r   s p e c i f i e d   a r g u m e n t s ------------ */
/* Argument Default       Description and remarks      Required/Optional: R/O */
/* -------- --------      --------------------------------------------------- */
,  Debug   = 0         /* Debug level; for testing, debugging purposes only O */
,  ForMyUse=           /* argument for the program's use only, don't use it O */
)/ DES     =              'Neat automatic ticmarks for graphs'  /*Store*/      ;
%*                                                                            *;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*;
%*                                                                            *;
%* File       : TickMark.SAS / TickMark.000                                   *;
%*                                                                            *;
%* Purpose    : Generation of neat, automatic tick marks for graphs (From,    *;
%*              To, By) based on minimum and maximum values of a data range.  *;
%*                                                                            *;
%* Programmer : Jim Groeneveld, OCS Biometric Support, `s Hertogenbosch, NL   *;
%* Date       : 25 Jul 2011                                                   *;
%* Version    : 0.0.1 / useful but still very much under construction         *;
%*                                                                            *;
%* SAS version: 9.1 under Windows XP (not (entirely) 8.x compatible anymore)  *;
%*                                                                            *;
%* Validation : -                                                             *;
%*                                                                            *;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*;
%*                                                                            *;
%* Summary:                                                                   *;
%* ========                                                                   *;
%*                                                                            *;
%* Theory:                                                                    *;
%* =======                                                                    *;
%*                                                                            *;
%* Used auxiliary macros:                                                     *;
%* ----------------------                                                     *;
%* = internal: -                                                              *;
%* = attached: -                                                              *;
%* = external: %ERR0R                                                         *;
%*                                                                            *;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*;
%*                                                                            *;
%* History of TickMark:                                                       *;
%* ====================                                                       *;
%* version  date         description of changes, additions, improvements      *;
%* -------  -----------  ---------------------------------------------------- *;
%* 0.0.0    14 Jul 2011  first working release, incl. Err0r & Warn1ng macros  *;
%* 0.0.1    25 Jul 2011  fixed bug with first main loop, improved last loop   *;
%*                                                                            *;
%* Future of TickMark:                                                        *;
%* ===================                                                        *;
%* vs. 0.0.2:                                                                 *;
%* - documentation, explain MinTicks and MaxTicks                             *;
%* - argument ZeroTick:                                                       *;
%* - anything, empty or whatever: as is, not necessarily including value 0    *;
%* - INCLUDE : explicitly include value 0 tick mark IF within value range     *;
%* - MIDDLE  : explicitly include value 0 as middle  tick mark *)             *;
%* - LOWEST  : explicitly include value 0 as lowest  tick mark *)             *;
%* - HIGHEST : explicitly include value 0 as highest tick mark *)             *;
%* *) specified coverage percentage may not be realised then                  *;
%*                                                                            *;
%******************************************************************************;

 %LOCAL    /* Define internal macro variables explicitely local */
           /* GENERAL */
  MacName  /* This macros external name: TickMark */
  MacVs    /* Version number major.minor.patchSub of macro: 0.0.1 */
  MacDate  /* Date of this version of macro: 25 Jul 2011 */
  ErrCount /* number of erroneously specified arguments */
  ;

  %LET MacName  = TickMark;
  %LET MacVs    = 0.0.1;
  %LET MacDate  = 25 Jul 2011;

  %IF ( %UPCASE(&ForMyUse) = COMPILE ) %THEN %GOTO Finish;

  %IF (&Debug EQ 1 OR &Debug GT 11) %THEN %DO;
    OPTIONS MPRINT MERROR SERROR MLOGIC SYMBOLGEN MACROGEN;
  %END;

%*~~~~~~~~~~~~~~~~~~~~~~~~~~;
%* Macro header in LOG file ;
%*__________________________;

  %PUT;
  %PUT |~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~|;
  %PUT |   Macro &MacName, vs. &MacVs, by Jim Groeneveld, &MacDate  |;
  %PUT | ===== Neat automatic ticmarks for graphs based on data ===== |;
  %PUT |   Version &Version: NOT yet VALIDATED, USE at YOUR own RISK!    |;
  %PUT |______________________________________________________________|;
  %PUT;

%* Feedback of specified arguments;
  %PUT &MacName: Review of argument specifications (incl. defaults);
  %PUT &MacName: Minimum  = &Minimum;
  %PUT &MacName: Maximum  = &Maximum;
  %PUT &MacName: FromVal  = &FromVal;
  %PUT &MacName: ToValue  = &ToValue;
  %PUT &MacName: ByValue  = &ByValue;
  %PUT &MacName: MinTicks = &MinTicks;
  %PUT &MacName: MaxTicks = &MaxTicks;
  %PUT &MacName: Coverage = &Coverage;
  %PUT &MacName: Version  = &Version;
  %PUT &MacName: Debug    = &Debug;

  %IF (&Version NE AND &Version NE &MacVs) %THEN %DO;
    %PUT *** &MacName *** &ERR: Specified version &Version does not match &MacVs,;
    %PUT .                       macro &MacName will abort;
    %GOTO Exit;
  %END;

%*~~~~~~~~~~~~~~~~~;
%* Argument checks ;
%*_________________;

%* Minimum;
  %LET ErrCount = 0;
  %LET Minimum = %SYSEVALF(&Minimum); %* Process any (complex) macro variable specification;
  %IF (NOT %LENGTH(&Minimum)) %THEN %DO;
    %PUT *** &MacName *** %ERR0R-: No minimum range value specified, is required;
    %LET ErrCount = %EVAL ( &ErrCount + 1);
  %END;
  %ELSE %IF (%SYSFUNC(VERIFY(&Minimum,'-.0123456789'))) %THEN %DO;
    %PUT *** &MacName *** %ERR0R-: No numeric minimum range value specified;
    %LET ErrCount = %EVAL ( &ErrCount + 1);
  %END;

%* Maximum (and related Minimum);
  %LET Maximum = %SYSEVALF(&Maximum); %* Process any (complex) macro variable specification;
  %IF (NOT %LENGTH(&Maximum)) %THEN %DO;
    %PUT *** &MacName *** %ERR0R-: No maximum range value specified, is required;
    %LET ErrCount = %EVAL ( &ErrCount + 1);
  %END;
  %ELSE %IF (%SYSFUNC(VERIFY(&Maximum,'-.0123456789'))) %THEN %DO;
    %PUT *** &MacName *** %ERR0R-: No numeric maximum range value specified (&Maximum);
    %LET ErrCount = %EVAL ( &ErrCount + 1);
  %END;
  %ELSE %IF (%SYSEVALF(&Minimum GE &Maximum)) %THEN %DO;
    %PUT *** &MacName *** %ERR0R-: Maximum range value (&Maximum) LE Minimum (&Minimum);
    %PUT .                        Maximum range value must be larger than Minimum one;
    %LET ErrCount = %EVAL ( &ErrCount + 1);
  %END;

/* - - - - - - - - -
%* FromVal, ToValue, ByValue;
  %IF (NOT %LENGTH(&FromVal)) %THEN %DO;
    %PUT *** &MacName *** %ERR0R-: No From-macro variable name specified, is required;
    %LET ErrCount = %EVAL ( &ErrCount + 1);
  %END;
  %IF (NOT %LENGTH(&ToValue)) %THEN %DO;
    %PUT *** &MacName *** %ERR0R-: No To-macro variable name specified, is required;
    %LET ErrCount = %EVAL ( &ErrCount + 1);
  %END;
  %IF (NOT %LENGTH(&ByValue)) %THEN %DO;
    %PUT *** &MacName *** %ERR0R-: No By-macro variable name specified, is required;
    %LET ErrCount = %EVAL ( &ErrCount + 1);
  %END;
- - - - - - - - - */

%* MinTicks;
  %LET MinTIcks = %EVAL(&MinTicks); %* Process any (complex) macro variable specification;
  %IF (NOT %LENGTH(&MinTicks)) %THEN %DO;
    %PUT *** &MacName *** %ERR0R-: No minimum number of tick marks specified, is required;
    %LET ErrCount = %EVAL ( &ErrCount + 1);
  %END;
  %ELSE %IF (%SYSFUNC(VERIFY(&MinTicks,'0123456789'))) %THEN %DO;
    %PUT *** &MacName *** %ERR0R-: No numeric minimum number of tick marks specified (&MinTicks);
    %LET ErrCount = %EVAL ( &ErrCount + 1);
  %END;
  %ELSE %IF (&MinTicks LT 2) %THEN %DO;
    %PUT *** &MacName *** %ERR0R-: Minimum number of tick marks too small (&MinTicks);
    %PUT .                        Minimum number of tick marks is 2;
    %LET ErrCount = %EVAL ( &ErrCount + 1);
  %END;

%* MaxTicks (and related MinTicks);
  %LET MaxTicks = %EVAL(&MaxTicks); %* Process any (complex) macro variable specification;
  %IF (NOT %LENGTH(&MaxTicks)) %THEN %DO;
    %PUT *** &MacName *** %ERR0R-: No maximum number of tick marks specified, is required;
    %LET ErrCount = %EVAL ( &ErrCount + 1);
  %END;
  %ELSE %IF (%SYSFUNC(VERIFY(&MaxTicks,'0123456789'))) %THEN %DO;
    %PUT *** &MacName *** %ERR0R-: No numeric maximum number of tick marks specified (&MaxTicks);
    %LET ErrCount = %EVAL ( &ErrCount + 1);
  %END;
  %ELSE %IF (&MinTicks GT &MaxTicks) %THEN %DO;
    %PUT *** &MacName *** %ERR0R-: Minimum number of tick marks (&MinTicks) GT maximum (&MaxTicks);
    %LET ErrCount = %EVAL ( &ErrCount + 1);
  %END;

%* Coverage;
  %LET Coverage = %SYSEVALF(&Coverage); %* Process any (complex) macro variable specification;
  %IF (NOT %LENGTH(&Coverage)) %THEN %DO;
    %PUT *** &MacName *** %ERR0R-: No percentage coverage specified, is required;
    %LET ErrCount = %EVAL ( &ErrCount + 1);
  %END;
  %ELSE %IF (%SYSFUNC(VERIFY(&Coverage,'0123456789'))) %THEN %DO;
    %PUT *** &MacName *** %ERR0R-: No numeric percentage coverage specified (&Coverage);
    %LET ErrCount = %EVAL ( &ErrCount + 1);
  %END;
  %ELSE %IF (%SYSEVALF(&Coverage LT 0 OR &Coverage GT 100)) %THEN %DO;
    %PUT *** &MacName *** %ERR0R-: Specified percentage Coverage (&Coverage) is out of range;
    %LET ErrCount = %EVAL ( &ErrCount + 1);
  %END;

%* ErrCount processing;
  %IF ( &ErrCount GT 0 ) %THEN %DO;
    %PUT .........aborting.........;
    %GOTO Exit;
  %END;

%*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~;
%* Local macro variable definitions ;
%*__________________________________;

 %LOCAL    /* Define internal macro variables explicitely local */
  I J      /* Counters */
  ValRange /* Range of values: maximum - minimum */
  LogRange /* 10th logarithm of ValRange (used only once) */
  Power    /* Power of 10 for initial (wide) tic mark units, CEIL of LogRange */
  From_Val /* Calculated From value, resulting minimum */
  To_Value /* Calculated To value, resulting maximum */
  By_Value /* Calculated increment */
  Factors  /* Multiplication factor list for powers of 10 */
  Multiply /* Multiplication factor from Factors list */
  LastMult /* Last Multiply before incremented */
  NewRange /* recalculated, widened and decreasing ValRange */
  NofUnits /* number of partial multiples of power of 10 */
  NofInts  /* adapted number of partial multiples of power of 10 */
  _NofInts /* starting NofInts */
  Ready    /* = 0, if (final) 100*ValRange/NewRange GE Coverage then Ready = 1 */
  ;
%* Unused: NofTicks Interval Decimals OldRange;

%*~~~~~~;
%* Core ;
%*______;

%*~~~~ALGORITHM~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~;
%* 0. Calculate original value range (ValRange)                                           ;
%* 1. Determine size of result ranges in terms of multiples of power of 10 of value range ;
%* 1a. - range rounded wide, repeat for certain partial multiples of power of 10 of range,;
%*       multiples are 1 .5 .2 .1 .05 .02 .01 .005 .002 .001 .                            ;
%* 1b. - once the wide rounded size of the result range becomes so small that Coverage of ;
%*       the original Range is covered by the new result range then stop decreasing size. ;
%* 2. Calculate From_Val (rounded down) and To_Value (rounded up) in terms of power of 10 ;
%*       of value range                                                                   ;
%* 3. Calculate NewRange from To_Value and From_Val                                       ;
%* 4. Calculate the final NewRange in terms of number of partial multiples of power of 10 ;
%*       as NofUnits (initial number of intervals).                                       ;
%* 5. If and while initial number of intervals is larger than specified maximum number    ;
%*       of tick marks (minus 1) group 2 or more partial multiples of power of 10 forming ;
%*       intervals, until number of intervals is equal or smaller than specified maximum  ;
%*       number of tick marks (minus 1),                                                  ;
%* 5a. - the To_Value and total NewRange may increase as a result of completing incomplete;
%*       intervals with partial multiples of power of 10. Choosen to decrease From_Val.   ;
%* 5b. - If ValRange/NewRange LT Coverage repeat/continue above loop for larger coverage  ;
%* 6. If and while initial number of intervals is smaller than specified maximum number   ;
%*       of tick marks (minus 1) split partial multiples of power of 10 into 2 or more    ;
%*       parts forming intervals, until number of intervals is equal or smaller than      ;
%*       specified maximum number of tick marks (minus 1),                                ;
%* 6a. - depending on the specified values for MinTicks and MaxTicks there may not be a   ;
%*       solution that meets the requirements.                                            ;
%*________________________________________________________________________________________;

%* Initialisations;
  %LET Factors = 1 .5 .2 .1 .05 .02 .01 .005 .002 .001; %* Multiplication factors;

%* 0. Calculate original value range (ValRange) ;
  %LET ValRange = %SYSEVALF (&Maximum - &Minimum);
%* 1. Determine size of result ranges in terms of multiples of power of 10 of value range;
  %LET LogRange = %SYSFUNC(LOG10(&ValRange));
  %LET Power = %SYSFUNC(CEIL(&LogRange));
  %IF (&Debug GT 0) %THEN %PUT ======================================================================;
  %IF (&Debug GT 0) %THEN %PUT DEBUG 0: LogRange=&LogRange, Power=&Power;
  %LET I = 1;
  %LET Multiply = %SCAN ( &Factors, &I, %STR( ) ); %* I=1, so Multiply=1;
  %LET Ready = 0;
  %DO %WHILE (NOT &Ready);
    %DO %UNTIL (%SYSEVALF((100*&ValRange/&NewRange) GE &Coverage));
%* 2. Calculate From_Val and To_Value in terms of power of 10 of value range;
      %LET From_Val = %SYSFUNC(ROUND (%SYSEVALF(&Minimum - &Multiply * .5 * 10**&Power),
                                    %SYSEVALF(&Multiply * 10**&Power)));
      %LET To_Value = %SYSFUNC(ROUND (%SYSEVALF(&Maximum + &Multiply * .5 * 10**&Power),
                                    %SYSEVALF(&Multiply * 10**&Power)));
%* 3. Calculate NewRange from To_Value and From_Val;
      %LET NewRange = %SYSEVALF(&To_Value-&From_Val);
      %IF (&Debug GT 0) %THEN %PUT DEBUG A: Multiply=&Multiply, ValRange=&ValRange, From_Val=&From_Val, To_Value=&To_Value, NewRange=&NewRange;
      %IF (&Debug GT 0) %THEN %PUT DEBUG B: I=&I, Multiply=&Multiply, Coverage=&Coverage, Percentage=%SYSEVALF(100*&ValRange/&NewRange);
      %IF (&Debug GT 0) %THEN %PUT DEBUG: ~~~~~~~~~ end of loop ~~~~~~~~~;
      %LET I = %EVAL ( &I + 1 );
      %LET Multiply = %SCAN ( &Factors, &I, %STR( ) );
    %END;

%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -;
%* 4. Calculate the final NewRange in terms of number of partial multiples of power of 10 ;
%*       as NofUnits (initial number of intervals).                                       ;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -;
    %LET LastMult = %SCAN ( &Factors, %EVAL(&I-1), %STR( ) ); %* I already became 1 more;
    %LET NofUnits = %SYSEVALF(&NewRange/(&LastMult * 10**&Power));

%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -;
%* 5. If and while initial number of intervals is larger than specified maximum number    ;
%*       of tick marks (minus 1) group 2 or more partial multiples of power of 10 forming ;
%*       intervals, until number of intervals is equal or smaller than specified maximum  ;
%*       number of tick marks (minus 1),                                                  ;
%* 5a. - the To_Value and total NewRange may increase as a result of completing incomplete;
%*       intervals with partial multiples of power of 10. Choosen to decrease From_Val.   ;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -;
    %LET J = 1;
    %LET NofInts = &NofUnits;
    %DO %WHILE (&NofInts GT %EVAL(&MaxTicks-1));
      %LET J = %EVAL ( &J + 1 );
      %LET NofInts = %SYSFUNC(CEIL(&NofUnits/&J));
    %END;
    %**LET To_Value = %SYSEVALF(&From_Val + &NofInts * &J * &LastMult * 10**&Power);
    %LET From_Val   = %SYSEVALF(&To_Value - &NofInts * &J * &LastMult * 10**&Power);
    %LET NewRange = %SYSEVALF(&To_Value-&From_Val);

    %LET By_Value = %SYSEVALF(&NewRange/&NofInts); %* = %SYSEVALF(&J * &LastMult * 10**&Power);
    %IF (&Debug GT 0) %THEN %PUT DEBUG C: J=&J, LastMult=&LastMult, NofUnits=&Nofunits, NofInts=&NofInts;
    %IF (&Debug GT 0) %THEN %PUT DEBUG D: To_Value=&To_Value, NewRange=&NewRange, By_Value=&By_Value;
    %IF (&Debug GT 0) %THEN %PUT DEBUG E: Coverage=&Coverage, Percentage=%SYSEVALF(100*&ValRange/&NewRange);

%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -;
%* 5b. - If ValRange/NewRange LT Coverage repeat/continue above loop for larger coverage  ;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -;
    %IF (%SYSEVALF((100*&ValRange/&NewRange) GE &Coverage)) %THEN %LET Ready = 1;
  %END;

%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -;
%* 6. If and while initial number of intervals is smaller than specified maximum number   ;
%*       of tick marks (minus 1) split partial multiples of power of 10 into 2 or more    ;
%*       parts forming intervals, until number of intervals is equal or smaller than      ;
%*       specified maximum number of tick marks (minus 1),                                ;
%* 6a. - depending on the specified values for MinTicks and MaxTicks there may not be a   ;
%*       solution that meets the requirements.                                            ;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -;
%* If the previous algorithm to decrease the number of intervals has been applied then no ;
%* need to apply the below algorithm de increase the number of intervals again: J GT 1    ;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -;
  %IF (&J EQ 1) %THEN %DO;
    %LET _NofInts = &NofInts;
    %LET Multiply = %SCAN ( &Factors, &J, %STR( ) ); %* J=1, so Multiply=1;
    %LET Ready = 0;
    %DO %UNTIL (&Ready);
      %LET NofInts = &_NofInts;
      %DO %WHILE (&NofInts LT %EVAL(&MinTicks-1) OR
        %SYSEVALF(&By_Value NE %SYSFUNC(ROUND(&By_Value,&Multiply * &LastMult * 10**&Power)))
        AND &NofInts LE %EVAL(&MaxTicks-1)); %* as long as not more than maximum;
%* By_Value may become value with many significant digits (decimals) if tick mark range too small;
/* ^ ^ ^ ^ ^ ^ ^ ^ ^
        %LET J = %EVAL ( &J + 1 );
        %LET NofInts = %EVAL(&NofUnits*&J);
v v v v v v v v v */
        %LET NofInts = %EVAL(&NofInts + 1);
        %LET By_Value = %SYSEVALF(&NewRange/&NofInts);
        %IF (&Debug GT 0) %THEN %PUT DEBUG F: Multiply=&Multiply, NofInts=&Nofints, By_Value=&By_Value, Rounded=%SYSFUNC(ROUND(&By_Value,&Multiply * &LastMult * 10**&Power));
      %END; %* macro DO WHILE;
      %IF (%SYSEVALF(&By_Value EQ %SYSFUNC(ROUND(&By_Value,&Multiply * &LastMult * 10**&Power))))
        %THEN %LET Ready = 1;
      %IF (&Debug GT 0) %THEN %PUT DEBUG G: J=&J, LastMult=&LastMult, NofUnits=&Nofunits, NofInts=&NofInts, Ready=&Ready;
      %IF (&Debug GT 0) %THEN %PUT DEBUG H: To_Value=&To_Value, NewRange=&NewRange, By_Value=&By_Value;
      %LET J = %EVAL ( &J + 1 );
      %LET Multiply = %SCAN ( &Factors, &J, %STR( ) );
    %END; %* macro DO UNTIL;
  %END; %* &J was 1;

  %PUT From_Val=&From_Val, To_Value=&To_Value, By_Value=&By_Value;

  %IF (%LENGTH(&FromVal) AND %LENGTH(&ToValue) AND %LENGTH(&ByValue)) %THEN %DO;
    %LET &FromVal = &From_Val;
    %LET &ToValue = &To_Value;
    %LET &ByValue = &By_Value;
  %END;
%* else return ORDER = statement contents (without [ORDER =] and without semicolon);
  %ELSE %IF (NOT &Debug) %THEN
    &From_Val TO &To_Value BY &By_Value; %* returned result if one of above not specified;

  %PUT *** &MacName *** INFO: Actual coverage percentage is: %SYSEVALF(100*&ValRange/&NewRange);

%Exit:
  %PUT ---------------------------------------------------------------;
  %PUT *** &MacName *** NOTE: End of macro &MacName ================= ;
  %PUT _______________________________________________________________;

%*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~;
%MEND TickMark;
%*___________________________________________________________________;
